#include "mesh.h"
#include "glut.h"

extern MODE mesh_display_mode; // grab the display mode
mesh::mesh()
{
	nX = nY = 0;
	grid2D = NULL;
}

mesh::mesh(int nx, int ny)
{
	Initialize(nx,ny);
}



// initialize th emesh;
void mesh::Initialize(int nx, int ny)
{
	nX = nx;
	nY = ny;

	int i,j;
	float pointX, pointY, texX, texY,
	      point_x_increment, point_y_increment,
	      tex_x_increment, tex_y_increment;
  
	// Allocate memory to our mesh
	// this is rowwise order, firtst allocate the rows and then fill the columns
	grid2D = (PointP2**)malloc(nY*sizeof(PointP2*));
	for(i=0; i<nY; i++)
		grid2D[i] = (PointP2*)malloc(nX*sizeof(PointP2));

	// now we assign coordinates
	// mesh ranges from -1 to 1 in both dimensions and
	// texture coordinates range from 0 to 1, tex(0,0) belong to point(-1,-1)
	// and tex(1,1) belong to point(1,1)
	// all points are uniformly distributed initially

	// we will fill the mesh bottom up
	pointX = LEFT;	// initialize our mesh points to lower left
	pointY = BOTTOM;
	texX = texY = 0.0;	// initialize texture coordinates similarly
	
	point_x_increment = (RIGHT-LEFT)/nX;	// all points are placed equidistant
	point_y_increment = (TOP-BOTTOM)/nY;
	tex_x_increment = 1.0/nX;
	tex_y_increment = 1.0/nY;
	
	for(i=0; i<nY; i++)	// for all the rows
	{
		for(j=0; j<nX; j++)	// for each column in a row
		{
			grid2D[i][j].point.row[0] = pointX;
			grid2D[i][j].point.row[1] = pointY;
			grid2D[i][j].point.row[2] = 1.0;	// homogeneous coordinates
			grid2D[i][j].tex.row[0] = texX;
			grid2D[i][j].tex.row[1] = texY;
			pointX += point_x_increment; // since a single row has same y coordinate
			texX += tex_x_increment;	   // we only adjust the x coordinates
		}	// end for each column
		pointY += point_y_increment;	// now we move to the next row
		texY += tex_y_increment;
		pointX = LEFT;	// reinitialize to beginning of the row
		texX = 0.0;
	}	// end for all rows
}	// end Initialize

void mesh::reset() //same stuff as above except malloc
{
	int i,j;
	float pointX, pointY, texX, texY,
	      point_x_increment, point_y_increment,
	      tex_x_increment, tex_y_increment;
	pointX = LEFT;	// initialize our mesh points to lower left
	pointY = BOTTOM;
	texX = texY = 0.0;	// initialize texture coordinates similarly
	
	point_x_increment = (RIGHT-LEFT)/nX;	// all points are placed equidistant
	point_y_increment = (TOP-BOTTOM)/nY;
	tex_x_increment = 1.0/nX;
	tex_y_increment = 1.0/nY;
	
	for(i=0; i<nY; i++)	// for all the rows
	{
		for(j=0; j<nX; j++)	// for each column in a row
		{
			grid2D[i][j].point.row[0] = pointX;
			grid2D[i][j].point.row[1] = pointY;
			grid2D[i][j].point.row[2] = 1.0;	// homogeneous coordinates
			grid2D[i][j].tex.row[0] = texX;
			grid2D[i][j].tex.row[1] = texY;
			pointX += point_x_increment; // since a single row has same y coordinate
			texX += tex_x_increment;	   // we only adjust the x coordinates
		}	// end for each column
		pointY += point_y_increment;	// now we move to the next row
		texY += tex_y_increment;
		pointX = LEFT;	// reinitialize to beginning of the row
		texX = 0.0;
	}	// end for all rows
}

void mesh::Clear()
{
	// this is to clean up the mesh
	// in case you want to create something that enables change in mesh size etc. at run time
	// so you can clean up and reinitialize with new settings
	// you can also modify mesh length and width by adjusting the #defines in mesh.h
	// or making them variables too

	if(grid2D == NULL) return; // grid is uninitialized to begin with

	int i;
	for(i=0; i<nY; i++)
	{
		free(grid2D[i]);
	}
	free(grid2D);
} // end Clear

int mesh::get_nX()	// get the number of vertices along X dimension
{
	return nX;
}

int mesh::get_nY()	// get the number of vertices along Y dimension
{
	return nY;
}

Vec3 mesh::getPoint(int x, int y)	// get the [x,y]th point's location
{
	return grid2D[x][y].point;
}

Vec2 mesh::getTex(int x, int y)	// get the [x,y]th point's texture coordinate
{
	return grid2D[x][y].tex;
}

int mesh::ApplyTransform(Mat3 H)	// Apply and H matrix to the points
{
	// note that transformation is only applied to the point coordinates
	// and not to the texture coordinates

	int i,j;
	//cout << "in ApplyTransform" << endl;
	//H.prnt();
	//getchar();
	for(i=0; i<nY; i++)
		for(j=0; j<nX; j++)
		{
			//grid2D[i][j].point.prnt();
			grid2D[i][j].point.mpy(H,grid2D[i][j].point);
			//grid2D[i][j].point.prnt();
		}
	return 1;
}

void mesh::display()
{
	// let us display just points first
	float x, y, tx, ty;
	glColor3f(1.0, 1.0, 1.0);
	if(mesh_display_mode==POINT)  // display mesh as points
	{
		glPointSize(1);
		glBegin(GL_POINTS);
		for(int i=0; i<nY; i++)
			for(int j=0; j<nX; j++)
			{
				x = grid2D[i][j].point.row[0]/grid2D[i][j].point.row[2];
				y = grid2D[i][j].point.row[1]/grid2D[i][j].point.row[2];
				tx = grid2D[i][j].tex.row[0];
				ty = grid2D[i][j].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);
			}
		glEnd();
	} // end points
	else if(mesh_display_mode==LINE) // display mesh as wireframe
	{
		glLineWidth(1);
		
		for(int i=0; i<nY-1; i++)
			for(int j=0; j<nX-1; j++)
			{
				glBegin(GL_LINE_STRIP);
				x = grid2D[i][j].point.row[0]/grid2D[i][j].point.row[2];
				y = grid2D[i][j].point.row[1]/grid2D[i][j].point.row[2];
				tx = grid2D[i][j].tex.row[0];
				ty = grid2D[i][j].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);

				x = grid2D[i+1][j].point.row[0]/grid2D[i+1][j].point.row[2];
				y = grid2D[i+1][j].point.row[1]/grid2D[i+1][j].point.row[2];
				tx = grid2D[i+1][j].tex.row[0];
				ty = grid2D[i+1][j].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);

				x = grid2D[i+1][j+1].point.row[0]/grid2D[i+1][j+1].point.row[2];
				y = grid2D[i+1][j+1].point.row[1]/grid2D[i+1][j+1].point.row[2];
				tx = grid2D[i+1][j+1].tex.row[0];
				ty = grid2D[i+1][j+1].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);

				x = grid2D[i][j+1].point.row[0]/grid2D[i][j+1].point.row[2];
				y = grid2D[i][j+1].point.row[1]/grid2D[i][j+1].point.row[2];
				tx = grid2D[i][j+1].tex.row[0];
				ty = grid2D[i][j+1].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);
				glEnd();
			}
		
	} // end wireframe
	else	// display mesh as filled polygons
	{
		glBegin(GL_QUADS);
		for(int i=0; i<nY-1; i++)
			for(int j=0; j<nX-1; j++)
			{
				x = grid2D[i][j].point.row[0]/grid2D[i][j].point.row[2];
				y = grid2D[i][j].point.row[1]/grid2D[i][j].point.row[2];
				tx = grid2D[i][j].tex.row[0];
				ty = grid2D[i][j].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);

				x = grid2D[i+1][j].point.row[0]/grid2D[i+1][j].point.row[2];
				y = grid2D[i+1][j].point.row[1]/grid2D[i+1][j].point.row[2];
				tx = grid2D[i+1][j].tex.row[0];
				ty = grid2D[i+1][j].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);

				x = grid2D[i+1][j+1].point.row[0]/grid2D[i+1][j+1].point.row[2];
				y = grid2D[i+1][j+1].point.row[1]/grid2D[i+1][j+1].point.row[2];
				tx = grid2D[i+1][j+1].tex.row[0];
				ty = grid2D[i+1][j+1].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);

				x = grid2D[i][j+1].point.row[0]/grid2D[i][j+1].point.row[2];
				y = grid2D[i][j+1].point.row[1]/grid2D[i][j+1].point.row[2];
				tx = grid2D[i][j+1].tex.row[0];
				ty = grid2D[i][j+1].tex.row[1];
				glTexCoord2f(tx,ty);
				glVertex2f(x,y);
			}
		glEnd();
	} // end quads
}
